package com.jhelplite;

import java.awt.BorderLayout;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Toolkit;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URL;
import java.util.Vector;

import javax.swing.JEditorPane;
import javax.swing.JFrame;
import javax.swing.JScrollPane;
import javax.swing.JSplitPane;
import javax.swing.JTree;
import javax.swing.SwingUtilities;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreePath;
import javax.swing.tree.TreeSelectionModel;


/**
 * Creates the Help Window and populates the contents.<BR>
 * <BR>
 * This window is created through the JHelpLightWindowFactory.getInstance() method.<BR>
 * <BR>
 * It is a JFrame, and can be manipulated as such.<BR>
 * <BR>
 * The text that is to be used as the help documents needs to be a HTML file, and can be in a package or
 * on the file system.<BR>
 * <BR>
 * The images that the tree uses for the folders, and leaves can be altered by creating a new TreeCellRenderer.<BR>
 * <BR>
 * The window icon (icon that is displayed in the top left corner of the frame) can be altered by setting the image icon.<BR>
 * 
 * @version 1.0
 * @author Gerald Arnold<BR>
 * Create Date: 09/March/2007<BR>
 * Copyright: 2007<BR>
 * Company: <COMPANY NAME><BR>
 */
public class JHelpLiteWindow
  extends JFrame
{
  private   static final long           serialVersionUID = 8665721380572340634L;

  private   JEditorPane                 editPaneContents;

  private   JScrollPane                 jscrollPaneMenu;
  private   JScrollPane                 jscrollPaneContents;

  private   JSplitPane                  splitPane;

  private   JTree                       treeHelpItems;
  private   DefaultTreeModel            treeModel;
  private   DefaultMutableTreeNode      rootNode;
  private   DefaultMutableTreeNode      selectedNode;
  private   TreeCellRenderer            treeCellRenderer;

  private   static String               FRAMETITLE         = "Help";
  protected static JHelpLiteUserObject ROOTNODEUSEROBJECT = new JHelpLiteUserObject();

  private int parentX;
  private int parentY;
  private int parentWidth;
  private int parentHeight;

  private static final int RIGHTPADDING  = 80; 
  private static final int LEFTPADDING   = 80;
  private static final int TOPPADDING    = 80;
  private static final int BOTTOMPADDING = 80;

  private Component parentComponent;



  /**
   * Default Constructor
   */
  protected JHelpLiteWindow()
  {
    parentX         = 0;
    parentY         = 0;
    parentWidth     = 0;
    parentHeight    = 0;

    parentComponent = null;

    ROOTNODEUSEROBJECT.setNodeName("Help");
  }



  /**
   * Default Constructor
   * 
   * @param parentComponent  The Parent Component of this window.
   */
  protected JHelpLiteWindow(Component parentComponent)
  {
    parentX              = parentComponent.getX();
    parentY              = parentComponent.getY();
    parentWidth          = parentComponent.getWidth();
    parentHeight         = parentComponent.getHeight();

    this.parentComponent = parentComponent;

    ROOTNODEUSEROBJECT.setNodeName("Help");
  }



  /**
   * Default Constructor
   * 
   * @param parentComponent  The Parent Component of this window.
   * @param frameTitle   The title for this frame.
   */
  protected JHelpLiteWindow(Component parentComponent, String frameTitle)
  {
    parentX              = parentComponent.getX();
    parentY              = parentComponent.getY();
    parentWidth          = parentComponent.getWidth();
    parentHeight         = parentComponent.getHeight();

    this.parentComponent = parentComponent;
    FRAMETITLE           = frameTitle;

    ROOTNODEUSEROBJECT.setNodeName("Help");
  }



  /**
   * Default Constructor
   * 
   * @param frameTitle  The title for this frame.
   */
  protected JHelpLiteWindow(String frameTitle)
  {
    parentX         = 0;
    parentY         = 0;
    parentWidth     = 0;
    parentHeight    = 0;

    parentComponent = null;
    FRAMETITLE      = frameTitle;

    ROOTNODEUSEROBJECT.setNodeName("Help");
  }




  /**
   * Sets the Parent Component to this window. This is used to set the position of this window to the middle of the parent.
   * 
   * @param parentComponent  The Parent Component of this window.
   */
  public void setParentComponent(Component parentComponent)
  {
    parentX              = parentComponent.getX();
    parentY              = parentComponent.getY();
    parentWidth          = parentComponent.getWidth();
    parentHeight         = parentComponent.getHeight();

    this.parentComponent = parentComponent;

    // Get the toolkit for the game
    Toolkit toolkit      = Toolkit.getDefaultToolkit();

    // Work out the current screensize
    Dimension screenSize = toolkit.getScreenSize();

    // Get the size of this frame
    Dimension frameSize  = this.getSize();

    // Get the position of this frame
    int x      = (int) (parentX + (parentWidth  / 2) - (frameSize.getWidth()  / 2));
    int y      = (int) (parentY + (parentHeight / 2) - (frameSize.getHeight() / 2));
    int width  = (int) frameSize.getWidth();
    int height = (int) frameSize.getHeight();

    // Make sure that this frame is not outside of the viewable area
    if (x + width > screenSize.getWidth())
    {
      x = (int) screenSize.getWidth() - width - RIGHTPADDING;
    }

    else if (x < 0)
    {
      x = LEFTPADDING;
    }


    if (y + height > screenSize.getHeight())
    {
      y = (int) screenSize.getHeight() - height - BOTTOMPADDING;
    }

    else if (y < 0)
    {
      y = TOPPADDING;
    }

    // Set it's X and Y location on the screen
    this.setLocation(x, y);

    return;
  }




  /**
   * Sets the text that is to be used in the title of this window.
   * 
   * @param frameTitle  The title for this frame.
   */
  public void setFrameTitle(String frameTitle)
  {
    FRAMETITLE = frameTitle;
    this.setTitle(frameTitle);

    return;
  }




  /**
   * Sets the TreeCellRenderer that will be used to draw each cell.
   * 
   * @param treeCellRenderer  The TreeCellRenderer that is to render each cell.
   */
  public void setCellRenderer(TreeCellRenderer treeCellRenderer)
  {
    this.treeCellRenderer = treeCellRenderer;

    if (this.treeCellRenderer != null && this.treeCellRenderer != null)
      treeHelpItems.setCellRenderer(this.treeCellRenderer);

    return;
  }




  /**
   * Gets the TreeCellRenderer that will be used to draw each cell.
   * 
   * @return TreeCellRenderer - The TreeCellRenderer that is to render each cell.
   */
  public TreeCellRenderer getCellRenderer()
  {
    if (treeCellRenderer != null)
      return treeCellRenderer;

    if (treeHelpItems != null)
      return treeHelpItems.getCellRenderer(); 

    return null;
  }




  /**
   * Set the size of the help window. If the help window has been initialised, then also set the preferred size of the JScrollPanes.
   * 
   * @param newWidth  The new Width to set the help window to.
   * @param newHeight The new Height to set the help window to.
   */
  public void setSize(int newWidth, int newHeight)
  {
    int width  = 300;
    int height = 400;

    if (newWidth > 300)
      width = newWidth;

    if (newHeight > 400)
      height = newHeight;

    if (jscrollPaneMenu != null)
      jscrollPaneMenu.setPreferredSize(new Dimension((int) (width * 0.33), height));

    if (jscrollPaneContents != null)
      jscrollPaneContents.setPreferredSize(new Dimension((int) (width * 0.67), height));

    super.setSize(width, height);

    return;
  }


  /**
   * Set the size of the help window. If the help window has been initialised, then also set the preferred size of the JScrollPanes.
   * 
   * @param newSize  The new size to set the help window to.
   */
  public void setSize(Dimension newSize)
  {
    int width  = 300;
    int height = 400;

    if (newSize.getWidth() > 300)
      width = (int) newSize.getWidth();

    if (newSize.getHeight() > 400)
      height = (int) newSize.getHeight();

    if (jscrollPaneMenu != null)
      jscrollPaneMenu.setPreferredSize(new Dimension((int) (width * 0.33), height));

    if (jscrollPaneContents != null)
      jscrollPaneContents.setPreferredSize(new Dimension((int) (width * 0.67), height));

    super.setSize(new Dimension(width, height));

    return;
  }




  /**
   * This method initialises all the elements used in this window.
   * 
   * @param rootNodeUserObject  The JHelpLightUserObject to use as the root node.
   */
  protected void init(JHelpLiteUserObject rootNodeUserObject)
  {
    ROOTNODEUSEROBJECT  = rootNodeUserObject;

    this.setTitle(FRAMETITLE);

    // For the menu side of things
    rootNode            = new DefaultMutableTreeNode((Object) ROOTNODEUSEROBJECT);
    treeModel           = new DefaultTreeModel(rootNode);
    treeHelpItems       = new JTree(treeModel);
    treeHelpItems.getSelectionModel().setSelectionMode(TreeSelectionModel.SINGLE_TREE_SELECTION);
    treeHelpItems.putClientProperty("JTree.linestyle", "Angled");

    if (treeCellRenderer != null)
      treeHelpItems.setCellRenderer(treeCellRenderer);

    jscrollPaneMenu     = new JScrollPane(treeHelpItems);
    jscrollPaneMenu.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    jscrollPaneMenu.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    jscrollPaneMenu.setWheelScrollingEnabled(true);
    jscrollPaneMenu.setAutoscrolls(true);
    jscrollPaneMenu.setDoubleBuffered(true);
    jscrollPaneMenu.setPreferredSize(new Dimension(250, 600));
    jscrollPaneMenu.setMinimumSize(new Dimension(100, 400));


    // For the contents side of things
    editPaneContents    = new JEditorPane();
    editPaneContents.setEditable(false);
    editPaneContents.setAutoscrolls(true);
    editPaneContents.setDoubleBuffered(true);

    jscrollPaneContents = new JScrollPane(editPaneContents);
    jscrollPaneContents.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    jscrollPaneContents.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
    jscrollPaneContents.setWheelScrollingEnabled(true);
    jscrollPaneContents.setAutoscrolls(true);
    jscrollPaneContents.setDoubleBuffered(true);
    jscrollPaneContents.setPreferredSize(new Dimension(550, 600));
    jscrollPaneContents.setMinimumSize(new Dimension(200, 400));


    // Now set the split plane
    splitPane       = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,
                                     jscrollPaneMenu,
                                     jscrollPaneContents);
    splitPane.setResizeWeight(0.40);

    return;
  }




  /**
   * This method builds the window.
   */
  protected void create()
  {
    // Catch the x in the top right being clicked
    this.addWindowListener(
      new WindowAdapter()
      {
        public void windowClosing(WindowEvent e)
        {
          dispose();
        }
      }
    );

    // The listener for the tree
    treeHelpItems.addMouseListener(
      new MouseAdapter()
      {
        public void mousePressed(MouseEvent e)
        {
          int selectedRow = treeHelpItems.getRowForLocation(e.getX(), e.getY());
          treeHelpItems.setSelectionRow(selectedRow);

          TreePath selectedPath = treeHelpItems.getSelectionPath();

          if (selectedPath != null)
          {
            selectedNode = (DefaultMutableTreeNode) selectedPath.getLastPathComponent();
            // Check to make sure that the left mouse button was pressed
            if (SwingUtilities.isLeftMouseButton(e))
            {
              populateHelpContents(((JHelpLiteUserObject) selectedNode.getUserObject()).getNodeDocumentLocation());
            }
          }
        }
      }
    );

    // Set the layout manager for this window
    this.getContentPane().setLayout(new BorderLayout());
    this.getContentPane().add(splitPane, BorderLayout.CENTER);

    this.setSize(800, 600);

    return;
  }




  /**
   * Set the Help Tree's Root Node's document location.
   * This document is the help page that will be displayed when the help window is first opened.
   * 
   * @param rootNodeName  The name to display for the root node.
   * @param rootNodeDocumentLocation  The location of the Root document.
   */
  protected void setRootNodeDocumentPath(String rootNodeName, String rootNodeDocumentLocation)
  {
    ROOTNODEUSEROBJECT.setNodeName(rootNodeName);
    ROOTNODEUSEROBJECT.setNodeDocumentLocation(rootNodeDocumentLocation);
    rootNode.setUserObject(ROOTNODEUSEROBJECT);

    return;
  }




  /**
   * Make the window visible. This need to be called after an instance of this class has been created to actually make the window appear.
   */
  public void makeWindowVisible()
  {
    // Pack the window
    this.pack();

    // Get the toolkit for the game
    Toolkit toolkit = Toolkit.getDefaultToolkit();

    // Work out the current screensize
    Dimension screenSize = toolkit.getScreenSize();

    // Get the size of this frame
    Dimension frameSize = this.getSize();

    int x = 0;
    int y = 0;
    if (parentComponent != null)
    {
      parentX              = parentComponent.getX();
      parentY              = parentComponent.getY();
      parentWidth          = parentComponent.getWidth();
      parentHeight         = parentComponent.getHeight();

      // Get the position of this frame
      x          = (int) (parentX + (parentWidth  / 2) - (frameSize.getWidth()  / 2));
      y          = (int) (parentY + (parentHeight / 2) - (frameSize.getHeight() / 2));
      int width  = (int) frameSize.getWidth();
      int height = (int) frameSize.getHeight();
  
      // Make sure that this frame is not outside of the viewable area
      if (x + width > screenSize.getWidth())
      {
        x = (int) screenSize.getWidth() - width - RIGHTPADDING;
      }
  
      else if (x < 0)
      {
        x = LEFTPADDING;
      }
  
  
      if (y + height > screenSize.getHeight())
      {
        y = (int) screenSize.getHeight() - height - BOTTOMPADDING;
      }
  
      else if (y < 0)
      {
        y = TOPPADDING;
      }
    }

    else
    {
      x = (int) (screenSize.getWidth()  / 2) - (int) (frameSize.getWidth()  / 2);
      y = (int) (screenSize.getHeight() / 2) - (int) (frameSize.getHeight() / 2);
    }

    // Set it's X and Y location on the screen
    this.setLocation(x, y);

    // Now make it visable
    this.setVisible(true);

    // Now load the root nodes text
    populateHelpContents(ROOTNODEUSEROBJECT.getNodeDocumentLocation());

    this.toFront();

    return;
  }




  /**
   * Close this window, and returns the focus to the parent window (the one that called this usually).
   */
  public void dispose()
  {
    super.dispose();
    if (parentComponent != null)
    {
      parentComponent.requestFocus();
      parentComponent.requestFocusInWindow();
    }

    return;
  }




  /**
   * Populated the index editor pane with the appropriate information.
   * 
   * @param indexNodes  A Vector containing JHelpLightUserObjects and Vectors that is used to populate the index.
   */
  @SuppressWarnings("unchecked")
  protected void populateHelpIndex(Vector indexNodes)
  {
    populteTreeFromVector(rootNode, indexNodes);

    treeHelpItems.expandRow(0);
    treeHelpItems.repaint();

    return;
  }




  /**
   * Populate a branch from the vector. This is a recursive method.
   * 
   * @param parent   The parent node of this branch.
   * @param subtree  A Vector containing JHelpLightUserObjects and Vectors that is used to populate the index.
   */
  @SuppressWarnings("unchecked")
  private void populteTreeFromVector(DefaultMutableTreeNode parent, Vector subtree)
  {
    DefaultMutableTreeNode parentNode = parent;

    for (int i = 0; i < subtree.size(); i++)
    {
      if (subtree.get(i) instanceof Vector)
      {
        populteTreeFromVector(parentNode, (Vector) subtree.get(i)); 
      }

      else
      {
        DefaultMutableTreeNode itemNode = new DefaultMutableTreeNode((JHelpLiteUserObject) subtree.get(i));

        parentNode.add(itemNode);

        if (i < (subtree.size() - 1))
        {
          if (subtree.get(i + 1) instanceof Vector)
          {
            i++;
            populteTreeFromVector(itemNode, (Vector) subtree.get(i));
          }
        }
      }
    }

    return;
  }




  /**
   * Populated the contents editor pane with the appropriate information.
   * The location of the document to use is stored in the tree nodes user object.
   * For the help window, the user object is a JHelpLightUserObject. In this object
   * the nodeDocumentLocation stores this value.
   * 
   * @param helpPageLocation  The location of the help page to use.
   */
  private void populateHelpContents(String helpPageLocation)
  {
    if (helpPageLocation != null && helpPageLocation.trim().length() > 0)
    {
      editPaneContents.setEditable(false);
      editPaneContents.setContentType("text/html");

      try
      {
        URL helpPageURL = this.getClass().getResource("/" + helpPageLocation.replaceAll("\\\\", "/")); 
        if (helpPageURL != null)
        {
          editPaneContents.setPage(helpPageURL);
        }

        else
        {
          editPaneContents.setPage(new File(helpPageLocation).toURL());
        }
      }

      catch(FileNotFoundException e)
      {
        editPaneContents.setEditorKit(null);
        editPaneContents.setText("Help file: " + helpPageLocation + " not found.");
      }

      catch(IOException e)
      {
        editPaneContents.setEditorKit(null);
        editPaneContents.setText("I/O error loading help file: " + helpPageLocation + ".");
        e.printStackTrace();
      }
    }

    return;
  }
}